import abc
from requests_html import HTML

class Filter(abc.ABC):
    @abc.abstractmethod
    def filter(parsed:dict):...
    

class CommonFilter(Filter):
    def __init__(self):
        self._parsed = None
        self._outputs = None

    def filter(self, parsed):
        self._parsed = parsed  
        self._outputs = parsed

        if hasattr(self, 'drops'):
            drop_items =  getattr(self, 'drops')() 
            if drop_items:
                if isinstance(drop_items, list):
                    for item in drop_items:
                        del self._outputs[item]
                else:
                    raise TypeError("Method Drops of {!r} returned a none-list object".format(self.__class__.__name__))

        if hasattr(self, 'keeps'):
            keep_items = getattr(self, 'keeps')()
            if keep_items:
                self._outputs = {} 
                if isinstance(keep_items, list):
                    for item in keep_items:
                        self._outputs[item] = self._parsed[item]
                else:
                    raise TypeError("Method Keeps of {!r} returned a none-list object".format(self.__class__.__name__))

        if hasattr(self, 'mutates'):
            mutate_items = getattr(self, 'mutates')(self._parsed)
            if mutate_items:
                if isinstance(mutate_items, dict):
                    self._outputs.update(mutate_items)
                else:
                    raise TypeError("Method mutates  of {!r} returned a none-dict object".format(self.__class__.__name__))
        return self._outputs

    def __call__(self, *args, **kwargs):
        return self.filter(*args, **kwargs)

    def __repr__(self):  # 这里需要改一下， 和xpather， 尽量使用子类自己的名字
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()

        
        
class MyFilter(CommonFilter):
    def __init__(self):
        ...

    def drops(self):  
        ...

    def keeps(self):
        ...
    
    def mutates(self, items):
        res = items['item1'] + items['item2']
        return [1]

    


if __name__ == "__main__":
    mf = MyFilter()
    result = mf.filter({"item1":"1", "item2": "2"})
    print(result)








        


import abc
from requests_html import HTML



class Cleaner(object):
    @staticmethod
    def clean(text:str):  
        return text.strip()



class Parser(abc.ABC):
    @abc.abstractmethod
    def rules():...

    @abc.abstractmethod
    def parse():...
    

class XpathParser(Parser):
    def __init__(self):
        self._rules = None
        self._result = {}

    def rules(self):
        self._rules = {attr:getattr(self, attr) for attr in dir(self) if not attr.startswith('_') and not callable(getattr(self, attr))}
        return self._rules

    def parse(self, html: HTML) -> dict:
        _ = self.rules()
        if self._rules:
            for item, rule in self._rules.items(): 
                self._result[item] = Cleaner.clean(html.xpath(rule,  first=True))
            return self._result
        return {}

    def __call__(self, *args, **kwargs):
        return self.parse(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()


#Parser for test  
class DmozParser(XpathParser):
    title = "//a[@class='logo']/span/text()"
    editors = "//div[@class='editors']/h3/text()[1]"

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()



if __name__ == "__main__":
    dp = DmozParser()



import os
import argparse
import configparser
import asyncio
from .engine import Engine
from .utils import configs_assertion
from .defaults import *

def fire():
    arg_parser = argparse.ArgumentParser()
    arg_parser.add_argument("configfile", type=str, 
        help="Please provide your configfile, then spydy will start soon.")
    args = arg_parser.parse_args()
    configfile = args.configfile
    if not os.path.exists(configfile):
        raise FileExistsError("No such file: {!r}".format(configfile))

    config_parser = configparser.ConfigParser()
    config_parser.read(configfile, encoding="utf-8")
    configs_assertion(config_parser)
    engine = Engine(config_parser)

    run_mode = config_parser["Globals"].get("run_mode", RUNMODE)   # choose run mode: once, forever, async: 
    if run_mode == "once":
        engine.run_once()
    if run_mode == "forever":
        engine.run_forever()
    if run_mode == "async":
        nworkers = int(config_parser["Globals"].get("nworkers", NWORKERS))
        loop = asyncio.get_event_loop()
        engine.run_async(loop, nworkers)



import abc
import csv
import asyncio

__all__ = ["CsvStore", "AsyncCsvStore", "StdOutStore"]

class Store(abc.ABC):
    @abc.abstractmethod
    def store():...


class StdOutStore(Store):
    def __init__(self):...

    def store(self, items:dict):
        print(items)

    def __call__(self, *args, **kwargs):
        self.store(*args, **kwargs)

    def __repr__(self):
        return "StdOutStore"

    def __str__(self):
        return self.__repr__()     


class CsvStore(Store):
    def __init__(self, file_name):
        self._filename = file_name

    def store(self, items:dict):
        fileds = list(items)
        with open(self._filename, 'a+', newline='') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=fileds)
            writer.writerow(items)

    def __call__(self, *args, **kwargs):
        self.store(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()     


class AsyncCsvStore(Store):
    def __init__(self, file_name):
        self.Async = ""
        self._filename = file_name

    async def store(self, items):
        items = await items
        fileds = list(items)
        with open(self._filename, 'a+', newline='') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=fileds)
            with await asyncio.Lock():
                writer.writerow(items)

    def __call__(self, *args, **kwargs):
        self.store(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()  

 


if __name__ == "__main__":
    cs = CsvStore(file_name="test.csv")
    cs.store(items={"a":"1", "b":"2"})


import configparser

config = configparser.ConfigParser()


config.read("spydy.cfg")

print(config.sections())

for arg, value in config["PipeLine"].items():
    print(arg)

print("PipeLine" in config)





import configparser

config = configparser.ConfigParser()

config['PipeLine'] = {
    'url':'RedisListUrls',
    'request':'LinearRequest',
    'parser' :'DmozParser',
    'store':'CscStore'
}


with open('config.ini', 'w') as configfile:
    config.write(configfile)


class TaskDone(Exception):
    """
      Indicate that the tasks have be finished gracefully.
    """

class UrlCompleted(TaskDone):
    """
        Taskd done, there are no more urls.
    """
    def __init__(self, msg):
        self.msg = msg

    def __repr__(self):
      return "UrlCompleted: {}".format(self.msg)




import abc
import redis
from .exceptions import UrlCompleted

__all__ = ["RedisListUrls"]


class Urls(abc.ABC):
    @abc.abstractmethod
    def pop():...

class RedisListUrls(Urls):
    def __init__(self, list_name, host="localhost", port=6379):
        self.list_name = list_name
        self._conn = redis.Redis(host=host, port=port)

    def pop(self):
        item = self._conn.lpop(self.list_name)
        if item:
            return item
        else:
            import sys
            sys.tracebacklimit = 0
            raise UrlCompleted("No value in redis list: {!r};Task will be shutdown in seconds..".format(self.list_name))
              
    def push(self, item):
        return self._conn.rpush(self.list_name, item)

    def complete(self):  
        print("No value in redis list: {!r}; Task will be shutdown in seconds...".format(self.list_name))


    def __call__(self, *args, **kwargs):
        return self.pop(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()
    



import abc
import aiohttp
import requests
from requests_html import HTML, AsyncHTMLSession

__all__ = ["LinearHttpGetRequest", "AsyncHttpGetRequest"]

class Request(abc.ABC):
    @abc.abstractmethod
    def get_html():...


class LinearHttpGetRequest(Request):
    def __init__(self, headers=None, proxy=None):   # 把mefhod去掉
        self._headers = headers
        if proxy:
            self._proxy = {
                'http': proxy,
                'https': proxy
            }
        else:
            self._proxy = None

    def get_html(self, url):
        with requests.session() as session:
            return HTML(html=session.get(url, headers=self._headers, proxies=self._proxy).text)

    def __call__(self, *args, **kwargs):
        return self.get_html(*args, **kwargs)

    def __repr__(self):
        #return "LinearRequest"
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()
    


class AsyncHttpGetRequest(Request):
    def __init__(self, headers=None, proxy=None):
        self.Async = ""
        self._headers = headers
        if proxy:
            self._proxy = {
                'http': proxy,
                'https': proxy
            }
        else:
            self._proxy = None

    async def get_html(self, url):
        url = url.decode("utf-8")
        asession = AsyncHTMLSession()
        response = await asession.get(url, headers=self._headers, proxies=self._proxy)
        return response.html

    def __call__(self, *args, **kwargs):
        return self.get_html(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return self.__repr__()



if __name__ == "__main__":
    from requests_html import HTML

    url = "https://www.dmoz-odp.org/"
    proxies = "http://2120070700089181521:5tGE2ivNSYwlkNs9@forward.apeyun.com:9082"
    lr = LinearHttpGetRequest(proxy=proxies)
    print(lr._proxy)
    response = lr.get_html(url)
    print(response.xpath("//a[@class='logo']/span/text()"))